/*
 * Copyright (c) 2018, Intel Corporation.
 * SPDX-License-Identifier: BSD-3-Clause
 */
#ifndef _FWVERSION_H_
#define _FWVERSION_H_

/** Minimum supported version of FW API: 1.2 **/
#define MIN_FIS_SUPPORTED_BY_THIS_SW_MAJOR   1
#define MIN_FIS_SUPPORTED_BY_THIS_SW_MINOR   2
/** Maximum supported version of FW API: 2.4 **/
#define MAX_FIS_SUPPORTED_BY_THIS_SW_MAJOR    2
#define MAX_FIS_SUPPORTED_BY_THIS_SW_MINOR    4

#endif /** _FWVERSION_H_ **/